// Copyright (c) 1997-2001, Sen:te (Sente SA).  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenRandomConformanceTest.h"

#define SEQUENCE_LENGTH 100L * 1000L

@implementation SenRandomConformanceTest
- (void) testSenRandomUnsignedLong
{
    unsigned long count = SEQUENCE_LENGTH;
    while (count--) {
        should ([generator nextUnsignedLong] == igenrand());
    }
}


- (void) testSenRandomDouble
{
    unsigned long count = SEQUENCE_LENGTH;
    while (count--) {
        double generated = [generator nextDoubleFrom0To1];
        should (generated == genrand());
        should ((generated >= 0.0) && (generated <= 1.0));
    }
}


- (void) testCokus
{
    unsigned long count = SEQUENCE_LENGTH;
    while (count--) {
        should (randomMT() == igenrand());
    }
}


- (void) testOptimizedSenRandom
{
    unsigned long count = SEQUENCE_LENGTH;
    SEL nextUnsignedLongSelector = @selector(nextUnsignedLong);
    unsigned long (*nextUnsignedLongFunction) (id, SEL);
    nextUnsignedLongFunction  = (unsigned long (*)(id, SEL)) [generator methodForSelector:nextUnsignedLongSelector];
    while (count--) {
        should (nextUnsignedLongFunction (generator, nextUnsignedLongSelector) == igenrand());
    }
}


- (void) testDice
{
    unsigned long count = SEQUENCE_LENGTH;
    while (count--) {
        unsigned int sideCount = 2 + [generator nextIntegerLessThan:99];
        should (sideCount <= 100);
        should ([generator nextIntegerLessThan:sideCount] < sideCount);
    }
}

@end
