//*$Id: SenTestTool.m,v 1.8 2001/12/13 08:27:22 alain Exp $*/
// Copyright (c) 1997-2000 Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenTestTool.h"
#import <SenFoundation/SenFoundation.h>

static NSString *SenTestedUnitPath = @"SenTestedUnitPath";
static NSString *SenTestScopeKey = @"SenTest";
static NSString *SenTestDefaultScope = @"Self";


@implementation SenTestTool

+ (void) initialize
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary *registeredDefaults = [NSDictionary dictionaryWithObjectsAndKeys:
        SenTestDefaultScope, SenTestScopeKey,
        nil];
    [defaults registerDefaults:registeredDefaults];
}


+ (SenTestTool *) sharedInstance
{
    static id instance = nil;
    if (instance == nil) {
        instance = [[self alloc] init];
    }
    return instance;
}


- (void) showUsage
{
    NSLog (@"Usage: %@ [-SenTest Self | All | None | <TestCaseClassName/testMethodName>] <path of unit to be tested>",
           [[NSProcessInfo processInfo] processName]);
}


- (NSBundle *) bundleWithPath:(NSString *) aPath
{
    if ([[NSFileManager defaultManager] fileExistsAtPath:aPath]) {
        return [NSBundle bundleWithPath:aPath];
    }
    return nil;
}


- (BOOL) runTestFromBundle:(NSBundle *) aBundle
{
    BOOL result = NO;

    [[NSUserDefaults standardUserDefaults] setObject:[aBundle bundlePath] forKey:SenTestedUnitPath];
    result = [aBundle load];
    if (result) {
        // The bundle has been loaded, and it should be linked against SenTestingKit.
        // so SenTestProbe should be available
        // (but we do not want to link otest with SenTestingKit to be able to test the kit itself.)
        id testProbeClass = NSClassFromString (@"SenTestProbe");
        if (testProbeClass != nil) {
            [testProbeClass performSelector:@selector(runTests:) withObject:nil];
        }
    }
    return result;
}


- (void) launchTaskFromPath:(NSString *) aPath bundle:(NSBundle *) aBundle
{
    NSString *testedUnitPath;
    NSString *launchPath;
    NSMutableArray *arguments = [NSMutableArray arrayWithArray:[[NSProcessInfo processInfo] arguments]];
    [arguments setArgumentDefaultValue:[[NSUserDefaults standardUserDefaults] stringForKey:SenTestScopeKey] forKey:SenTestScopeKey];

    if (aBundle != nil) {
        testedUnitPath = aPath;
        launchPath = [aBundle executablePath];
    }
    else {
        // FIXME: This is a hack based on the fact that [NSBundle bundleForClass:aClass] returns a bundle with
        // the tool parent folder as its path.
        testedUnitPath = [aPath stringByDeletingLastPathComponent];
        launchPath = aPath;
    }
    [arguments setArgumentDefaultValue:testedUnitPath forKey:SenTestedUnitPath];
    [[NSTask launchedTaskWithLaunchPath:launchPath arguments:arguments] waitUntilExit];
}


- (void) run
{
    NS_DURING
        if ([[[NSProcessInfo processInfo] arguments] count] < 2) {
            [NSException raise:NSInvalidArgumentException format:@"Missing arguments"]; 
        }
        else {
            NSString *path = [[[NSProcessInfo processInfo] arguments] lastObject];
            NSBundle *bundle = [self bundleWithPath:path];
            if (bundle != nil) {
                if (![self runTestFromBundle:bundle]) {
                    [self launchTaskFromPath:path bundle:bundle];
                }
            }
            else {
                [self launchTaskFromPath:path bundle:nil];
            }
        }
            
    NS_HANDLER
        [self showUsage];
        NSLog (@"%@", localException);
    NS_ENDHANDLER
}


+ (void) run
{
   [[self sharedInstance] run];
}
@end
